@@ -1,6 +1,6 @@
 #
 #  This file is part of the Connection-Set Algebra (CSA).
-#  Copyright (C) 2010,2011,2012 Mikael Djurfeldt
+#  Copyright (C) 2010,2011,2012,2019 Mikael Djurfeldt
 #
 #  CSA is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,12 +19,13 @@
 import math
 import random
 import copy
+#from scipy.spatial import KDTree
 
-import connset as cs
-import valueset as vs
-import _elementary
+from . import connset as cs
+from . import valueset as vs
+from . import _elementary
 
-from csaobject import *
+from .csaobject import *
 
 class Random (cs.Operator):
     def __mul__ (self, valueSet):
@@ -59,8 +60,8 @@ class ValueSetRandomMask (cs.Mask):
         return self
 
     def iterator (self, low0, high0, low1, high1, state):
-        for j in xrange (low1, high1):
-            for i in xrange (low0, high0):
+        for j in range (low1, high1):
+            for i in range (low0, high0):
                 if random.random () < self.valueSet (i, j):
                     yield (i, j)
 
@@ -83,8 +84,8 @@ class DiscMask (cs.Mask):
         self.metric = metric
 
     def iterator (self, low0, high0, low1, high1, state):
-        for j in xrange (low1, high1):
-            for i in xrange (low0, high0):
+        for j in range (low1, high1):
+            for i in range (low0, high0):
                 if self.metric (i, j) < self.r:
                     yield (i, j)
 
@@ -111,8 +112,8 @@ class RectangleMask (cs.Mask):
         self.g1 = g1
 
     def iterator (self, low0, high0, low1, high1, state):
-        for j in xrange (low1, high1):
-            for i in xrange (low0, high0):
+        for j in range (low1, high1):
+            for i in range (low0, high0):
                 p0 = self.g0 (i)
                 p1 = self.g1 (j)
                 dx = p0[0] - p1[0]
@@ -123,6 +124,7 @@ class RectangleMask (cs.Mask):
  
 class Gaussian (cs.Operator):
     def __init__ (self, sigma, cutoff):
+        cs.Operator.__init__ (self, 'gaussian')
         self.sigma = sigma
         self.cutoff = cutoff
         
@@ -162,41 +164,107 @@ class BlockMask (cs.Mask):
         self.N = N
         self.m = mask
 
+    def startIteration (self, state):
+        #*fixme* filter out 'partitions' from state
+        nState = {}
+        for k in state:
+            if k != 'partitions':
+                nState[k] = state[k]
+        self.obj = self.m.startIteration (nState)
+        return self
+
     def iterator (self, low0, high0, low1, high1, state):
-        maskIter =  self.m.iterator (low0 / self.M,
-                                     (high0 + self.M - 1) / self.M,
-                                     low1 / self.N,
-                                     (high1 + self.N - 1) / self.N,
-                                     state)
+        maskIter =  self.obj.iterator (low0 // self.M,
+                                       (high0 + self.M - 1) // self.M,
+                                       low1 // self.N,
+                                       (high1 + self.N - 1) // self.N,
+                                       state)
         try:
             pre = []
-            (i, j) = maskIter.next ()
+            (i, j) = next (maskIter)
             while True:
                 # collect connections in one connection matrix column
                 post = j
                 while j == post:
                     pre.append (i)
-                    (i, j) = maskIter.next ()
+                    (i, j) = next (maskIter)
 
                 # generate blocks for the column
-                for jj in xrange (max (self.N * post, low1),
+                for jj in range (max (self.N * post, low1),
                                   min (self.N * (post + 1), high1)):
                     for k in pre:
-                        for ii in xrange (max (self.M * k, low0),
+                        for ii in range (max (self.M * k, low0),
                                           min (self.M * (k + 1), high0)):
                             yield (ii, jj)
                 pre = []
         except StopIteration:
             if pre:
                 # generate blocks for the last column
-                for jj in xrange (max (self.N * post, low1),
+                for jj in range (max (self.N * post, low1),
                                   min (self.N * (post + 1), high1)):
                     for k in pre:
-                        for ii in xrange (max (self.M * k, low0),
+                        for ii in range (max (self.M * k, low0),
                                           min (self.M * (k + 1), high0)):
                             yield (ii, jj)
 
 
+class Repeat (cs.Operator):
+    def __init__ (self, M, N):
+        self.M = M
+        self.N = N
+
+    def __mul__ (self, other):
+        c = cs.coerceCSet (other)
+        if isinstance (c, cs.Mask):
+            return RepeatMask (self.M, self.N, c)
+        else:
+            return cs.ConnectionSet (RepeatCSet (self.M, self.N, c))
+
+
+# Not fully implemented
+# Currently only handles cases where we iterate over an even number of
+# ocurrences of the template mask, both with regard to sources and targets
+#
+class RepeatMask (cs.Mask):
+    def __init__ (self, M, N, mask):
+        cs.Mask.__init__ (self)
+        self.M = M
+        self.N = N
+        self.m = mask
+
+    def iterator (self, low0, high0, low1, high1, state):
+        jj = low1
+        nextHigh1 = (low1 + self.N) / self.N * self.N
+        while nextHigh1 <= high1:
+            maskIter =  self.m.iterator (0,
+                                         self.M,
+                                         0,
+                                         self.N,
+                                         state)
+            try:
+                (i, j) = next (maskIter)
+                post = j
+                while post < self.N:
+                    pre = []
+                    while j == post:
+                        pre.append (i)
+                        (i, j) = next (maskIter)
+                    ii = low0
+                    while ii < high0:
+                        for k in pre:
+                            yield (ii + k, jj + post)
+                        ii += self.M
+                    post = j
+            except StopIteration:
+                ii = low0
+                while ii < high0:
+                    for k in pre:
+                        yield (ii + k, jj + post)
+                    ii += self.M
+            jj = nextHigh1
+            nextHigh1 += self.N
+
+
 class Transpose (cs.Operator):
     def __mul__ (self, other):
         c = cs.coerceCSet (other)
@@ -235,7 +303,7 @@ class FixedMask (cs.FiniteMask):
         for c in mask:
             ls.append (c)
         self.connections = ls
-        targets = map (cs.target, ls)
+        targets = list (map (cs.target, ls))
         self.low0 = min (ls)[0]
         self.high0 = max (ls)[0] + 1
         self.low1 = min (targets)
